package controller.web;
import java.io.IOException;
import java.util.HashMap;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@SuppressWarnings("serial")
public class FrontController extends HttpServlet {

	private static final HashMap<String, Action> allCMD = new HashMap<String, Action> ();

	static {
		allCMD.put("0.login", new LoginAction());
		allCMD.put("0.register", new RegisterAction());
		allCMD.put("1.homeInput", new SearchCreateNoteAction());
		allCMD.put("1.logout", new LogoutAction());
		allCMD.put("1.home", new HomeAction());
		allCMD.put("1.deleteNote", new DeleteNoteAction());
		allCMD.put("1.saveNote", new SaveNoteAction());
		allCMD.put("1.showNote", new ShowNoteAction());
		allCMD.put("1.editNote", new EditNoteAction());
	}

	public static final HashMap<String, String> allFiles = new HashMap<String, String>();

	static {
		allFiles.put("login", "/jsp/Login.jsp");
		allFiles.put("register", "/jsp/Register.jsp");
		allFiles.put("home", "/jsp/Home.jsp");
		allFiles.put("showNote", "/jsp/ShowNote.jsp");
		allFiles.put("editNote", "/jsp/EditNote.jsp");
		allFiles.put("searchResultNote", "/jsp/SearchResultNote.jsp");
	}
	static ResourceBundle rb_de = ResourceBundle.getBundle("gui.ResourcesNotemanager", Locale.GERMAN);
	static ResourceBundle rb_en = ResourceBundle.getBundle("gui.ResourcesNotemanager", Locale.ENGLISH);

	private void handleRequest(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
		determineLanguage(req);

		String cmd = req.getParameter("cmd");
		String username = (String) req.getSession().getAttribute("username");
		System.out.println("\n\n--> FrontController handleRequest");
		System.out.println("--cmd "+ cmd);

		Action action = (Action) getRequest(username, cmd, req);

		if(action != null) {
			String nextFile = action.execute(req, res);
			System.out.println("--file: /jsp/" + nextFile);

			if(nextFile != null || !nextFile.equals("")) {
				getServletContext().getRequestDispatcher(allFiles.get(nextFile)).forward(req, res);
			}
		}
	}

	private void determineLanguage(HttpServletRequest req) {
		String index = "," + req.getHeader("accept-language");
		int ide = index.indexOf(",de");
		int ien = index.indexOf(",en");
		if(ide < ien){
			req.setAttribute("resBundle", rb_de);
		}else{
			req.setAttribute("resBundle", rb_en);
		}
	}

	private Action getRequest(String username, String cmd, HttpServletRequest req) {
		String actionCommand = "";
		if (username != null && !username.equals("")) {
			if (allCMD.get(1 + "." + cmd) != null) {
				actionCommand = "1."+ cmd;
			}
			else {
				actionCommand = "1.home";
			}

		} else {				
			if(cmd != null && cmd.equals("register")){
				actionCommand = "0.register";
			}else{
				actionCommand = "0.login";
			}

		}
		return (Action) allCMD.get(actionCommand);
	}
	
	protected void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
		handleRequest(req, res);
	}
	protected void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
		handleRequest(req, res);
	}
}
