/*
 * Name: LoginAction.java
 * Version: 1.0
 * Beschreibung: Die Klasse LoginAction.java, die nach login.jsp aufgerufen wird
 * und dazu dient, den Benutzer einzuloggen.
 * 
 */
package controller.web;

import java.util.Collection;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import model.Category;
import model.ModelImplementation;
import model.ModelInterface;

public class LoginAction extends Action {

	public String execute(HttpServletRequest request,
			HttpServletResponse response) {

		String username = (String) request.getParameter("username");
		String password = (String) request.getParameter("password");
		HashMap<String, String> errorMap = new HashMap<String, String>();

		if (completelyFilledIn(username, password)) {
			if (isLoginCorrect(request, username, password, errorMap)) {
				ModelInterface mi = ModelImplementation.getInstance();
				Collection<Category> categoryList = mi.getCategoryList();
				request.setAttribute("categoryList", categoryList);
				request.getSession().setAttribute("categoryList",categoryList );
				request.getSession().getAttribute("categoryList");
				
				return "home";
			}
		} else {
			handleEmptyFields(request, errorMap);
		}
		request.setAttribute("errorMap", errorMap);
		return "login";
	}

	/**
	 * Gibt nur eine Warnung wenn Seite nicht zum ersten Mal aufgerufen wird.
	 */
	private void handleEmptyFields(HttpServletRequest request, HashMap<String, String> errorMap) {
		if (request.getParameter("loginButton") != null) {
			errorMap.put("loginErr", "Username und/oder Passwort leer.");
			errorMap.put("username", "true");
			errorMap.put("pw", "true");
		}
	}

	private boolean isLoginCorrect(HttpServletRequest request, String username, String password, HashMap<String, String> errorMap) {
		ModelInterface mi = ModelImplementation.getInstance();

		if (mi.startDoLogin(username, password)) {
			request.getSession().setAttribute("username", username);
			return true;
		} else {
			errorMap.put("loginErr", "Der Benutzername existiert nicht " +
			"oder das Passwort wurde falsch geschrieben!");
			errorMap.put("username", "true");
			errorMap.put("pw", "true");
			return false;
		}
	}

	private boolean completelyFilledIn(String username, String password) {
		return username != null && password != null && !username.equals("")
		&& !password.equals("");
	}
}
