package controller.web;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import model.ModelImplementation;
import model.ModelInterface;

public class RegisterAction extends Action{

	@Override
	public String execute(HttpServletRequest request,
			HttpServletResponse response) {

		String email = (String) request.getParameter("email");
		String username = (String) request.getParameter("username");
		String password = (String) request.getParameter("password");
		String password2 = (String) request.getParameter("password2");

		if(request.getParameter("rButton") != null){
			
			if (completelyFilledIn(email, username, password, password2)) {
				registerUser(username, password);
				return "login";

			} else {
				handleError(request, email, username, password, password2);
			}
		}
		return "register";
	}

	private void handleError(HttpServletRequest request, String email, String username, String password, String password2) {
		HashMap<String, String> errorMap = new HashMap<String, String>();
		if(checkIfFieldIsNotEmpty(password) && checkIfFieldIsNotEmpty(password2) && !comparePasswords(password, password2)) {
			errorMap.put("registerErr", "Die eingegebenen Passwörter stimmen nicht überein!");
			errorMap.put("username", "true");
			errorMap.put("pw1", "true");
			errorMap.put("pw2", "true");
		}
		else {
			errorMap.put("registerErr", "Felder nicht ausgefüllt.");
			distinguishFields(email, username, password, password2, errorMap);
		}
		request.setAttribute("errorMap", errorMap);
	}

	private void distinguishFields(String email, String username, String password, String password2, HashMap<String, String> errorMap) {
		if(!checkIfFieldIsNotEmpty(email)){
			errorMap.put("email", "true");
		}
		if(!checkIfFieldIsNotEmpty(username)){
			errorMap.put("username", "true");
		}
		if(!checkIfFieldIsNotEmpty(password)){
			errorMap.put("pw1", "true");
		}
		if(!checkIfFieldIsNotEmpty(password2)){
			errorMap.put("pw2", "true");
		}
	}

	private void registerUser(String username, String password) {
		ModelInterface mi = ModelImplementation.getInstance();
		mi.startDoRegistration(username, password);
	}

	private boolean completelyFilledIn(String email, String username, String password, String password2) {
		return checkIfFieldIsNotEmpty(username) && checkIfFieldIsNotEmpty(password) && checkIfFieldIsNotEmpty(email) &&
			   comparePasswords(password, password2);
	}
	
	private boolean checkIfFieldIsNotEmpty(String field) {
		return field != null && !field.equals("");
	}
	
	private boolean comparePasswords(String pw1, String pw2) {
		return pw1.equals(pw2);
	}
}
