/*
 * Name: SearchCreateNoteAction.java
 * Version: 1.0
 * Beschreibung: Die Klasse SearchCreateNoteAction.java, die nach home.jsp (jenachdem ob
 * Erstellen oder Suchen aufgerufen wird) dient dazu, dass eine neue Notiz gespeichert
 * bzw. nach Notizen gesucht wird.
 * 
 */
package controller.web;


import java.util.Collection;
import java.util.Date;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import model.ModelImplementation;
import model.ModelInterface;
import model.Note;
import model.SearchCriteria;

public class SearchCreateNoteAction extends Action {

	public String execute(HttpServletRequest request, HttpServletResponse response) {

		Note noteObject = new Note();
		ModelInterface mi = ModelImplementation.getInstance();
		request.getSession().setAttribute("actualNote", null);
		
		if (request.getParameter("action") == null) {
			return "home";
		}
		else {
			if (request.getParameter("action").equals("create")){
				return create(request, noteObject, mi);
			} else {
				return search(request, mi);
			}
		}
	}

	private String create(HttpServletRequest request, Note noteObject,
			ModelInterface mi) {
		fillNoteObject(request, noteObject, mi, true);
		
		if ((!noteObject.getTitle().equals("") || !noteObject.getFullText().equals(""))) {
			noteObject.setDBID(mi.saveNote(noteObject));
			request.getSession().setAttribute("actualNote", noteObject);
			return "showNote";
		}
		else {
			handleError(request, noteObject);
			request.setAttribute("categoryList", mi.getCategoryList());
			return "home";
		}
	}

	private void handleError(HttpServletRequest request, Note noteObject) {
		HashMap<String, String> errorMap = new HashMap<String, String>();
		request.getSession().setAttribute("actualNote", noteObject);
		errorMap.put("createSearchErr", "Titel und/oder Text wurde(n) nicht ausgef&uuml;llt.");
		errorMap.put("title", "true");
		errorMap.put("text", "true");
		request.setAttribute("errorMap", errorMap);
	}

	private String search(HttpServletRequest request, ModelInterface mi) {
		SearchCriteria sc = null;
		if (request.getParameter("ishomeinput") != null) {
			sc = fillSearchCriteria(request);
		}
		else {
			sc = (SearchCriteria) request.getSession().getAttribute("searchCriteria");
		}
		Collection<Note> searchResults = mi.searchNote(sc);
		request.setAttribute("searchResult", searchResults);
		return "searchResultNote";
	}

	private SearchCriteria fillSearchCriteria(HttpServletRequest request) {
		SearchCriteria sc;
		sc = new SearchCriteria();
		sc.setAuthor(request.getParameter("author"));
		sc.setCategories((Collection<?>) request.getAttribute("categories"));
		sc.setDateFrom(request.getParameter("dateFromField"));
		sc.setDateTo(request.getParameter("dateToField"));
		sc.setFullText(request.getParameter("fullText"));
		sc.setKeyword(request.getParameter("keyword"));
		sc.setTitle(request.getParameter("title"));
		if (request.getParameter("categoryList") != null) {
			sc.setSingleCategoryID(Integer.parseInt(request.getParameter("categoryList")));			
		}
		sc.setCheckedAllCategories(null != request.getParameter("checkAllCategories"));
		request.getSession().setAttribute("searchCriteria", sc);
		return sc;
	}

//	private void fillNoteObject(HttpServletRequest request, Note noteObject, ModelInterface mi, boolean bool) {
//		noteObject.setTitle(request.getParameter("title"));
//		noteObject.setFullText(request.getParameter("fullText"));
//		noteObject.setKeyword(request.getParameter("keyword"));
//		noteObject.setDate(new Date());
//		String username = (String) request.getSession().getAttribute("username");
//		noteObject.setAuthor(username);
//		noteObject.setAuthorID(mi.getUserIDByUsername(username));
//		noteObject.setVersionID(Integer.parseInt(request.getParameter("version")));
//		noteObject.setDBID(Integer.parseInt(request.getParameter("DBID")));
//		request.getSession().setAttribute("actualNote", noteObject);
//	}
}
