package sql;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;


public class DBConnectionPool {
	
	Connection con;
	Connection temp;
	
	private Vector<Connection> freeConnections;
	private Vector<Connection> busyConnections;
	private int maxConnections;
	private String username;
	private String password;
	private boolean autocommit;
	private String dbUrl;
	
	public DBConnectionPool(int _maxConnections, String _username, String _password,
							boolean _autocommit, String _dbUrl){
		
		freeConnections = new Vector<Connection>();
		busyConnections = new Vector<Connection>();
		maxConnections  = _maxConnections;
		username		= _username;
		password		= _password;
		autocommit		= _autocommit;
		dbUrl			= _dbUrl;
		

		//check if username and password is set
		for(int i=0; i<maxConnections; i++){
			if(pwSet(username, password)){
				loadDriverWithPassword();
			}
			else{
				loadDriver();
			}
			System.out.println("dbConnectionPool; "+ con);
			freeConnections.add(con);
		}
	}

	private void loadDriver() {
		try{
			con = DriverManager.getConnection(dbUrl);
			con.setAutoCommit(autocommit);
		}
		catch(SQLException ex){
			System.out.println("Fehler: " + ex.getMessage());
		}
	}

	private void loadDriverWithPassword() {
		try{
			con = DriverManager.getConnection(dbUrl+"?" + 
		                   					"user="+username+"&password="+password+"&charSet=UTF8");
			con.setAutoCommit(autocommit);
		}
		catch(SQLException ex){
			System.out.println("Fehler: " + ex.getMessage());
		}
	}

	private boolean pwSet(String username, String Password) {
		if(username != null && username != ""){
			if(password != null && password != ""){
				return true;
			}
		}		
		return false;
	}
	
	public Connection getConnection(){
		Connection temp = (Connection) freeConnections.get(0);
		freeConnections.remove(0);
		busyConnections.add(temp);
		return temp;
	
	}
	
	public void putConnection(Connection con){
		freeConnections.add(con);
		busyConnections.remove(con);
	}
	
    @SuppressWarnings("unchecked")
	protected void closeAll() throws Throwable {
        Collection c = new ArrayList(freeConnections);
        c.addAll(busyConnections);
        for (Iterator i = c.iterator(); i.hasNext();) {
            Connection con =  (Connection) i.next();
            try{
                
            con.close(); 
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}
