package sql;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;

import model.Note;
import model.SearchCriteria;

public class SearchDAO extends DAO{

	String query = "SELECT DISTINCT " +
				   "UserID, Nickname, NoteID, VersionID, Title, Author, Text, " +
				   "Date, Keyword, IsNewest, FK_NoteID, FK_CategoryID, CategoryID " +
				   "FROM user, noteversion, category, note_category WHERE noteversion.IsNewest = '1' " +
				   "AND UserID = Author AND FK_NoteID = NoteID AND CategoryID = FK_CategoryID";
	Connection c;
	
	@SuppressWarnings("unchecked")
	public Collection<Note> searchNote(SearchCriteria sc) {
		c = this.getConnection();
		try{
			stmt = createStatementSearch(sc, c);
			System.out.println("SQL Statement "+ query);
			ResultSet dbResult = stmt.executeQuery(query + " ORDER BY Title");
			Collection searchResult = new ArrayList();
            System.out.println("result " + dbResult.toString());
			while(dbResult.next()){
				fillSearchResult(dbResult, searchResult);
			}
			c.setAutoCommit(false);
			c.commit();
			c.setAutoCommit(true);
			stmt.close();
			ArrayList a = (ArrayList) searchResult;
			System.out.println("Size of Result : "+ a.size());
			return searchResult;
		}
		catch(SQLException ex){
			System.out.println("SQL ex: " + ex);
			errorQuery(c, ex);
			return null;
		}
		finally{
			this.putConnection(c);
		}
	}

	private void fillSearchResult(ResultSet dbResult, Collection<Note> searchResult) throws SQLException {
		Note nv = new Note();
		nv.setAuthor(dbResult.getString("Nickname"));
		nv.setTitle(dbResult.getString("Title"));
		nv.setDate(dbResult.getTimestamp("Date"));
		//nv.setDate(dbResult.getDate("Date"));
//		System.out.println("searchdao fillsearch... "+ nv.getDateString());
		nv.setFullText(dbResult.getString("Text"));
		nv.setKeyword(dbResult.getString("Keyword"));
		nv.setVersionID(dbResult.getInt("VersionID"));
		//NoteID ist DBID
		nv.setDBID(dbResult.getInt("NoteID"));
		nv.setSingleCategoryID(dbResult.getInt("FK_CategoryID"));
		searchResult.add(nv);
	}

	private Statement createStatementSearch(SearchCriteria sc, Connection c) throws SQLException {
		c = this.getConnection();
		Statement stmt;
		
		addTerm(sc.getAuthor(), "user.Nickname LIKE", true);
		addTerm(sc.getFullText(), "Text LIKE ", true);
		addTerm(sc.getKeyword(), "Keyword LIKE ", true);
		addTerm(sc.getTitle(), "Title LIKE ", true);
		if(sc.isCheckedAllCategories()){
			System.out.println("Test Kategorie: 1");
			//addTerm("*", "FK_CategoryID =", true);
		}else{
			addTerm(sc.getSingleCategoryID(), "FK_CategoryID = ", true);
		}
		validateDate(sc);
		

		stmt = c.createStatement();
		this.putConnection(c);
		return stmt;
	}

	private void validateDate(SearchCriteria sc) {

		String dateFrom = sc.getDateFrom()+ " 00:00:00";
		String dateTo = sc.getDateTo()+ " 23:59:59";
		
		System.out.println(dateFrom);
		System.out.println(dateTo);
		if(dateFrom.equals("yyyy-MM-dd 00:00:00") && dateTo.equals("yyyy-MM-dd 23:59:59")){
		}
		else if(dateTo.equals("yyyy-MM-dd 23:59:59") || sc.getDateTo().equals("")){
			System.out.println("blablablubb");
			addTerm(dateFrom, "Date >= ");
		}
		else if(dateFrom.equals("yyyy-MM-dd 00:00:00") || sc.getDateFrom().equals("")){
			addTerm(dateTo, "Date <= ");
		}
		else {
			System.out.println("ELSE");
			addTerm(dateFrom, "Date >= ");
			addTerm(dateTo, "Date <= ");
		}
	}

	private void addTerm(Object term, String queryDiff){
		if(null != term && term.toString().length() > 0){
			query += " AND ";
			query += queryDiff + " " + "\"" + term.toString() + "\"";
		}
	}
	
	private void addTerm(Object term, String queryDiff, boolean addLIKE){
		if(null != term){
			addTerm(term.toString().replace('"', '%'), queryDiff);
		}
	}

	public Note getSimpleNote(int noteID, int versionID){
		c = this.getConnection();
		
		try {
			
			String query = "SELECT DISTINCT " +
			   "* " +
			   "FROM noteversion, note_category WHERE " +
			   " FK_NoteID = NoteID AND NoteID = "+noteID+" AND VersionID = "+ versionID +"";
			
			stmt = c.createStatement();
			System.out.println("Query "+ query);
			//String query = "SELECT * FROM noteversion WHERE NoteID = "+noteID+" AND VersionID = "+ versionID +" ";
			System.out.println("NoteID: " + noteID + " versionID: " + versionID);
			ResultSet dbResult = stmt.executeQuery(query);
			
			Note nv = new Note();
			while(dbResult.next()){
			nv.setAuthor(getAuthorByID(dbResult.getInt("Author")));
			nv.setTitle(dbResult.getString("Title"));
			nv.setDate(dbResult.getTimestamp("Date"));
			nv.setFullText(dbResult.getString("Text"));
			nv.setKeyword(dbResult.getString("Keyword"));
			nv.setVersionID(dbResult.getInt("VersionID"));
			//NoteID ist DBID
			nv.setSingleCategoryID(dbResult.getInt("FK_CategoryID"));
			nv.setDBID(dbResult.getInt("NoteID"));
			
			return nv;
			}
			
			return null;
			
		} catch (SQLException e) {
			errorQuery(c, e);
			return null;
		}finally{
			this.putConnection(c);
		}
				
	}
	
	public String getAuthorByID(int AuthorID){
		c = this.getConnection();
		try {
			stmt = c.createStatement();
			String query = "SELECT Nickname FROM user WHERE UserID = "+AuthorID+"";
			ResultSet dbResult = stmt.executeQuery(query);
			
			while(dbResult.next()){
				return dbResult.getString("Nickname");
			}
			
			return null;
			

			
		} catch (SQLException e) {
			errorQuery(c, e);
			return null;
		}finally{
			this.putConnection(c);
		}
	}

	public static void main(String[] args) {
		SearchCriteria sc = new SearchCriteria();
		sc.setAuthor("fi*");
		new SearchDAO().searchNote(sc);

	}

}

