package tests;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

import junit.framework.Assert;
import model.AbstractNote;
import model.ModelImplementation;
import model.ModelInterface;
import model.Note;
import model.SearchCriteria;

import org.junit.Test;

import sql.DBConnectionManager;
import static org.junit.Assert.*;

public class NoteTests {
	
	ModelInterface mi = ModelImplementation.getInstance();
	Note n = new Note();
	static Note noteAct;

	@Test public void login(){
		assertTrue(mi.startDoLogin("valentin.groetsch", "12345"));
		assertFalse(mi.startDoLogin("b", "b"));
	}
	
	@Test public void createNote() throws SQLException{
		fillData(n);
		n.setAuthorID(3);
		n.setVersionID(1000);
		n.setDBID(0);
		n.setIsNewest(1);
		
		assertTrue(mi.saveNote(n) > 0);
	}
	
	@Test public void searchNoteCreated(){
		Collection actual = fillSearchCriteria();
		assertTrue(actual.size() == 1);
		
		Iterator itAct = actual.iterator();
		int j = 0;
		noteAct = null;
		while(itAct.hasNext() && j == 0)
		{
			j++;
			noteAct = (Note) itAct.next();
		}
		assertEquals("JUnit", noteAct.getTitle());
		assertEquals("fabian.schumann", noteAct.getAuthor());
		assertEquals("JUnit Test Create", noteAct.getFullText());
	}

	@Test public void deleteNote(){
		assertTrue(mi.setNoteInactive(noteAct.getDBID()));
	}
	
	@Test public void searchNoteDeleted(){
		Collection actual = fillSearchCriteria();
		assertTrue(actual.size() < 1);
	}
	
	private Collection fillSearchCriteria() {
		SearchCriteria sc = new SearchCriteria();
		fillData(sc);
		sc.setDateFrom(null);
		sc.setDateTo(null);
		
		Collection actual = mi.searchNote(sc);
		return actual;
	}
	
	private void fillData(AbstractNote an) {
		an.setTitle("JUnit");
		an.setKeyword("unit, test");
		an.setFullText("JUnit Test Create");
		an.setAuthor("fabian.schumann");
		an.setCategories(null);
	}
}
